@@ -56,6 +56,10 @@ class Config:
         pickle.dump(config, self.CONFIG_FILE.open('wb'))
 
 
+class ApiException(Exception):
+    pass
+
+
 class Tapd:
     def __init__(self, iteration_id=None, base_url=None):
         self.base_url = base_url or TAPD_BASE_URL
@@ -91,6 +95,15 @@ class Tapd:
                 response.json().get('data', [])]
         return OrderedDict(data)
 
+    def get_tapd_test_plans(self):
+        """custom_field_2 为迭代 ID"""
+        url = urljoin(self.base_url, '/tapd/origin/test_plans')
+        response = requests.get(url,
+                                params={'status': 'open', 'custom_field_2': self.iteration_id, 'fields': 'id,name'})
+        data = [(item['TestPlan']['name'], item['TestPlan']['id']) for item in
+                response.json().get('data', [])]
+        return OrderedDict(data)
+
     def post_tapd_task(self, **kwargs):
         """
         创建或更新任务
@@ -103,7 +116,7 @@ class Tapd:
         response = requests.post(url, data=kwargs)
         json_data = response.json()
         if not json_data.get('data'):
-            raise Exception('用例上传失败, 原因：', json_data.get('data', 'no data'))
+            raise ApiException('用例上传失败, 原因：', json_data.get('info', 'no data'))
 
         task_id = json_data.get('data', {}).get('Task', {}).get('id')
         return task_id
@@ -122,11 +135,22 @@ class Tapd:
         response = requests.post(url, data=kwargs)
         json_data = response.json()
         if not json_data.get('data'):
-            raise Exception('用例上传失败, 原因：', json_data.get('data', 'no data'))
+            raise ApiException('用例上传失败, 原因：', json_data.get('info', 'no data'))
 
         case_id = json_data.get('data', {}).get('Tcase', {}).get('id')
         return case_id
 
+    def post_tapd_test_plans_and_tcase_relation(self, **kwargs):
+        """
+        创建测试计划和测试用例关联关系
+        teet_plan_id, tcase_ids, creator
+        """
+        url = urljoin(self.base_url, '/tapd/origin/test_plans/create_tcase_relation')
+        response = requests.post(url, data=kwargs)
+        json_data = response.json()
+        if json_data.get('status') != 1:
+            raise ApiException('创建测试计划和测试用例关联关系失败, 原因：', json_data.get('info', 'status != 1'))
+
 
 class Excel:
     def __init__(self, filename, tapd: Tapd = None):
@@ -231,15 +255,30 @@ class Cases:
         self.iteration_meta_info = self.excel.get_iteration_meta_info()
         self.title_column_letters = self.excel.case_title_column_letters
 
+    def is_valid_case(self, *required_cell):
+        """
+        判断是否是有效的用例
+        功能用例：`一级模块`	`用例名称`
+        冒烟用例：`一级模块`	`用例名称`	`用例等级（必须是 高）`	`开发人员`	`需求`
+        """
+        for cell in required_cell:
+            if not self.case_ws[cell].value:
+                return False
+        else:
+            return True
+
     def validate_data(self):
+        """
+        功能用例：`一级模块`	`用例名称` 必填
+        """
         results = []
         cases_count = 0
         for row_index in range(3, self.case_ws.max_row + 1):
-            case_title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
+            title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
             custom_field_1_cell = f'{self.title_column_letters["一级模块"]}{row_index}'
-            if self.case_ws[case_title_cell].value:
-                if not self.case_ws[custom_field_1_cell].value:
-                    results.append(f'第 {row_index} 行用例【一级模块】不能为空')
+            if self.case_ws[title_cell].value:
+                if not self.is_valid_case(custom_field_1_cell):
+                    results.append(f'第 {row_index} 行用例【一级模块】/【用例名称】不能为空')
                 else:
                     cases_count += 1
         return results, cases_count
@@ -252,6 +291,7 @@ class Cases:
 
         iteration_name = self.iteration_meta_info.get('iteration_name')
         categories_map = self.excel.tapd.get_tapd_case_categories()
+        test_plans_map = self.excel.tapd.get_tapd_test_plans()
 
         def html_format(content: str):
             return '<br />'.join(content.strip().splitlines())
@@ -279,40 +319,58 @@ class Cases:
                 steps_cell = f'{self.title_column_letters["用例步骤"]}{row_index}'
                 expectation_cell = f'{self.title_column_letters["预期结果"]}{row_index}'
                 priority_cell = f'{self.title_column_letters["用例等级"]}{row_index}'
+                story_cell = f'{self.title_column_letters["需求"]}{row_index}'
                 case_id_cell = f'{self.title_column_letters["case_id"]}{row_index}'
-                case_id = self.excel.tapd.post_tapd_case(
-                    id=self.case_ws[case_id_cell].value or None,
-                    category_id=categories_map.get(self.case_ws[custom_field_1_cell].value, 'None Category'),
-                    name=self.case_ws[title_cell].value,
-                    precondition=html_format(self.case_ws[precondition_cell].value or ''),
-                    steps=html_format(self.case_ws[steps_cell].value or ''),
-                    expectation=html_format(self.case_ws[expectation_cell].value or ''),
-                    priority=html_format(self.case_ws[priority_cell].value or ''),
-                    creator=creator,
-                    custom_field_1=self.case_ws[custom_field_1_cell].value or '',
-                    custom_field_2=self.case_ws[custom_field_2_cell].value or '',
-                    custom_field_5=iteration_name,
-                )
 
-                if case_id:
-                    self.case_ws[case_id_cell].value = case_id
+                # 只上传合法的用例
+                if self.is_valid_case(custom_field_1_cell, title_cell):
+                    case_id = self.excel.tapd.post_tapd_case(
+                        id=self.case_ws[case_id_cell].value or None,
+                        category_id=categories_map.get(self.case_ws[custom_field_1_cell].value, 'None Category'),
+                        name=self.case_ws[title_cell].value,
+                        precondition=html_format(self.case_ws[precondition_cell].value or ''),
+                        steps=html_format(self.case_ws[steps_cell].value or ''),
+                        expectation=html_format(self.case_ws[expectation_cell].value or ''),
+                        priority=html_format(self.case_ws[priority_cell].value or ''),
+                        creator=creator,
+                        custom_field_1=self.case_ws[custom_field_1_cell].value or '',
+                        custom_field_2=self.case_ws[custom_field_2_cell].value or '',
+                        custom_field_5=iteration_name,
+                    )
+
+                    if case_id:
+                        # 创建测试计划和测试用例关联关系, 已关联过的用例不重复关联
+                        test_plan_id = test_plans_map.get(self.case_ws[story_cell].value)
+                        if test_plan_id and not self.case_ws[case_id_cell].value:
+                            self.excel.tapd.post_tapd_test_plans_and_tcase_relation(
+                                test_plan_id=test_plan_id,
+                                tcase_ids=case_id,
+                                creator=creator,
+                            )
+
+                        # 写入用例 ID
+                        self.case_ws[case_id_cell].value = case_id
 
 
 class BVTCases(Cases):
     def validate_data(self):
+        """
+        冒烟用例：`一级模块`	`用例名称`	`用例等级（必须是 高）`	`开发人员`	`需求`
+        """
         results = []
         cases_count = 0
         for row_index in range(3, self.case_ws.max_row + 1):
-            case_title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
-            if self.case_ws[case_title_cell].value:
-                priority_cell = f'{self.title_column_letters["用例等级"]}{row_index}'
-                if self.case_ws[priority_cell].value == '高':  # 冒烟用例
-                    story_cell = f'{self.title_column_letters["需求"]}{row_index}'
-                    developer_cell = f'{self.title_column_letters["开发人员"]}{row_index}'
-                    if not all([self.case_ws[developer_cell].value, self.case_ws[story_cell].value]):
-                        results.append(f'第 {row_index} 行冒烟用例【开发人员】和【需求】不能为空')
-                    else:
-                        cases_count += 1
+            priority_cell = f'{self.title_column_letters["用例等级"]}{row_index}'
+            title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
+            if self.case_ws[title_cell].value and self.case_ws[priority_cell].value == '高':  # 冒烟用例
+                custom_field_1_cell = f'{self.title_column_letters["一级模块"]}{row_index}'
+                developer_cell = f'{self.title_column_letters["开发人员"]}{row_index}'
+                story_cell = f'{self.title_column_letters["需求"]}{row_index}'
+                if not self.is_valid_case(custom_field_1_cell, developer_cell, story_cell):
+                    results.append(f'第 {row_index} 行冒烟用例【一级模块】/【用例名称】/【开发人员】/【需求】不能为空')
+                else:
+                    cases_count += 1
+
         return results, cases_count
 
     def upload(self, creator):
@@ -334,9 +392,9 @@ class BVTCases(Cases):
         ) as bar:
             time.sleep(0.5)  # 强制先暂停 0.5s, 避免太快进度条没刷出来就结束了，体验提升 50%
             for row_index in bar:
-                developer_cell = f'{self.title_column_letters["开发人员"]}{row_index}'
                 priority_cell = f'{self.title_column_letters["用例等级"]}{row_index}'
                 if self.case_ws[developer_cell].value and self.case_ws[priority_cell].value == '高':
+                    developer_cell = f'{self.title_column_letters["开发人员"]}{row_index}'
                     custom_field_1_cell = f'{self.title_column_letters["一级模块"]}{row_index}'
                     custom_field_2_cell = f'{self.title_column_letters["二级模块"]}{row_index}'
                     title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
@@ -346,20 +404,22 @@ class BVTCases(Cases):
                     story_cell = f'{self.title_column_letters["需求"]}{row_index}'
                     task_id_cell = f'{self.title_column_letters["task_id"]}{row_index}'
 
-                    task_id = self.excel.tapd.post_tapd_task(
-                        id=self.case_ws[task_id_cell].value or None,
-                        name=f'【冒烟用例】{self.case_ws[title_cell].value}',
-                        creator=creator,
-                        owner=self.case_ws[developer_cell].value,
-                        story_id=stories_map.get(self.case_ws[story_cell].value),
-                        iteration_id=self.iteration_meta_info['iteration_id'],
-                        description=template.render(
-                            custom_field_1=self.case_ws[custom_field_1_cell].value or '',
-                            custom_field_2=self.case_ws[custom_field_2_cell].value or '',
-                            precondition=self.case_ws[precondition_cell].value or '',
-                            case_steps=self.case_ws[steps_cell].value or '',
-                            expectation=self.case_ws[expectation_cell].value or ''
+                    # 只上传合法的冒烟用例
+                    if self.is_valid_case(custom_field_1_cell, title_cell, developer_cell, story_cell):
+                        task_id = self.excel.tapd.post_tapd_task(
+                            id=self.case_ws[task_id_cell].value or None,
+                            name=f'【冒烟用例】{self.case_ws[title_cell].value}',
+                            creator=creator,
+                            owner=self.case_ws[developer_cell].value,
+                            story_id=stories_map.get(self.case_ws[story_cell].value),
+                            iteration_id=self.iteration_meta_info['iteration_id'],
+                            description=template.render(
+                                custom_field_1=self.case_ws[custom_field_1_cell].value,
+                                custom_field_2=self.case_ws[custom_field_2_cell].value or '',
+                                precondition=self.case_ws[precondition_cell].value or '',
+                                case_steps=self.case_ws[steps_cell].value or '',
+                                expectation=self.case_ws[expectation_cell].value or ''
+                            )
                         )
-                    )
-                    if task_id:
-                        self.case_ws[task_id_cell].value = task_id
+                        if task_id:
+                            self.case_ws[task_id_cell].value = task_id
